function scn= MSpAng2Q1(sco, NameAi, NameDel, NameGam)
% function scnn= MSpAng2Q1(scno, NameAi, NameDel, NameGam)
% Transform angles to Q-coordinates for 
%	GID/GISAX on a liquid interface at ID10B, i.e., sample
%	plane is horizontata (liquid), primary beam deflected from the
%	horizontal plane by the deflection crystal by an angle mu (i.e.
%	mu= angle of incidence), detector is rotated around the vertical
%	axis by angle del, detector is rotated around the horizontal axis
%	by angle gam, gam sits on del, PSD detector is oriented along
%	the vertical axis Morotrs. 
% For ID10B ESRF morotrs names: NameAi='mu', NameDel='delta',
% NameGam='gamma'
% (c) 2007, Jiri Novak, ESRF Grenoble, France 
%%%%
% This program comes with ABSOLUTELY NO WARRANTY.  This is free
% software distributed under GNU licence, see
% http://www.gnu.org/licenses/ or the attached file GPLicense.txt



d2r=pi/180;
ai=sco.(NameAi)*d2r;
del=sco.(NameDel)*d2r;
gam0=sco.(NameGam)*d2r;
cpd=sco.cpd;
c0=sco.c0;
channels=sco.channs;
nChannels=size(sco.PSD,2);
nai=length(ai);
ndel=length(del);
ngam0=length(gam0);
nmax=max([nai,ndel,ngam0]);
if (nai<nmax),
	if (nai~=1) error('Wrong number of scan points for %s\n',NameAi);
	end;
	ai=repmat(ai,nmax,1);
end;
if (ndel<nmax),
	if (ndel~=1) error('Wrong number of scan points for %s\n',NameDel);
	end;
	del=repmat(del,nmax,1);
end;
if (ngam0<nmax),
	if (ngam0~=1),
		error('Wrong number of scan points for %s\n',NameGam);
	end;
	gam0=repmat(gam0,nmax,1);
end;
gamd=atan((channels-c0)*d2r/cpd);
csgam0=cos(gam0); sngam0=sin(gam0);
csgamd=cos(gamd); sngamd=sin(gamd);
csgam=csgam0*csgamd-sngam0*sngamd;
sngam=csgam0*sngamd+sngam0*csgamd;
csdel=cos(del); sndel=sin(del);
csai=cos(ai); snai=sin(ai);
if size(csai,2)==1,
	csai=repmat(csai,1,nChannels);
	snai=repmat(snai,1,nChannels);
end;


Qx=repmat(csdel,1,nChannels).*csgam-csai;
Qy=repmat(sndel,1,nChannels).*csgam;
Qz=sngam+snai;
scn=sco;
K0=2*pi/sco.lam;
scn.Qx=Qx*K0;
scn.Qy=Qy*K0;
scn.Qz=Qz*K0;

