function [Qxg,delQx,Qyg,delQy,Qzg,delQz]=MSpEstimQGrid(sco,xGridField,yGridField)
% MSpEstimQGrid
% Estimation of optimal grid-points for gridding of Qx, Qy, Qz
% experimental data for purpose of 2D plotting of intensity in Q-space.
% Furthemore, the function estimates optimal Qx, Qy, Qz sizes of the bins around the
% grid-points for purposes of the gridding by the algorithm used in the
% program gridq.cc.
% Additionaly, the function prints out suggested parameters on terminal.
% The suggested grid parameters can be past into the main matlab script
% for plotting the data (that which call this function). It is advisable
% to modify the grid parameters further to get an optimal grid.
% Usage:
% [Qxg,delQx,Qyg,delQy,Qzg,delQz]=MSpEstimQGrid(sco,xGridField,yGridField);
% Input: 
%		sco ... a structure with scan data
%		xGridField ... name of the variable, which will be on the x-axis
%			(i.e. 'Qx', 'Qy' or 'Qz')
%		yGridField ... name of the variable, which will be on the y-axis
%			(i.e. 'Qx', 'Qy' or 'Qz')
% Output:
%		Qxg, Qyg, Qzg ... estimated optimal grid-points 
%   delQx, delQy, delQz ... estimated optimal witdth of the bins around
%   the grid-points
% (c) 2007, Jiri Novak, ESRF Grenoble, France 
%%%%
% This program comes with ABSOLUTELY NO WARRANTY.  This is free
% software distributed under GNU licence, see
% http://www.gnu.org/licenses/ or the attached file GPLicense.txt



Qx=sco.Qx;
Qy=sco.Qy;
Qz=sco.Qz;

[Qxr,Qxd]=detDimRang(Qx); delQx=Qxr(2)-Qxr(1);
[Qyr,Qyd]=detDimRang(Qy); delQy=Qyr(2)-Qyr(1);
[Qzr,Qzd]=detDimRang(Qz); delQz=Qzr(2)-Qzr(1);

vars=[xGridField(2),yGridField(2)];
errControl=0;
[mQ,ii]=sort([delQx,delQy,delQz]);
stepC=1.2; binC=1.5;
fprintf(1,'Suggested ranges of the grid density and range in Q space:\n')
fprintf(1,'To be copy and paste into the matlab script:\n')
if ~isempty(strfind(vars,'x')),
	fprintf(1,'Qxg=[%g:%g:%g]; delQx=%g;\n', Qxr(1), stepC*Qxd, Qxr(2),binC*Qxd);
	Qxg=[Qxr(1):stepC*Qxd:Qxr(2)]; delQx=binC*Qxd;
	errControl=errControl+1;
else
	fprintf(1,'Qxg=[%g]; delQx=%g;\n',mean(Qxr),2*(Qxr(2)-Qxr(1)));
	Qxg=[mean(Qxr)]; delQx=2*delQx;
end;
if ~isempty(strfind(vars,'y')),
	fprintf(1,'Qyg=[%g:%g:%g]; delQy=%g;\n', Qyr(1), stepC*Qyd, Qyr(2),binC*Qyd);
	Qyg=[Qyr(1):stepC*Qyd:Qyr(2)]; delQy=binC*Qyd;
	errControl=errControl+1;
else
	fprintf(1,'Qyg=[%g]; delQy=%g;\n',mean(Qyr),2*(Qyr(2)-Qyr(1)));
	Qyg=[mean(Qyr)]; delQy=2*delQy;
end;
if ~isempty(strfind(vars,'z')),
	fprintf(1,'Qzg=[%g:%g:%g]; delQz=%g;\n', Qzr(1), stepC*Qzd, Qzr(2),binC*Qzd);
	Qzg=[Qzr(1):stepC*Qzd:Qzr(2)]; delQz=binC*Qzd;
	errControl=errControl+1;
else
	fprintf(1,'Qzg=[%g]; delQz=%g;\n',mean(Qzr),2*(Qzr(2)-Qzr(1)));
	Qzg=[mean(Qzr)]; delQz=2*delQz;
end;
if (errControl~=2),
	error('Wrong input parameters xGridField and yGridField');
end;

function mi=minmin(m)
	mi=min(min(m));

function ma=maxmax(m)
	ma=max(max(m));

function mmi=meanmin(m)
	mmi=mean(min(m));

function mma=meanmax(m)
	mma=mean(max(m));

function meandel=meandif(m)
	meandel=mean( mean( abs( diff(m))));

function [sugQ,sugDel,dir]=detDimRang(m);
% sugQ=[Qmin,Qmax] ... suggested ranges
% dir ... dominant direction of the change of M, 1... along columns,
%			... 2 along rows
	mmiv=meanmin(m); mmav=meanmax(m);
	mmih=meanmin(m'); mmah=meanmax(m');
	delv=mmav-mmiv; delh=mmah-mmih;
	sugQ=[0,0];
	if delv>delh,
		sugQ(1)=mmiv-std(min(m)); 
		sugQ(2)=mmav+std(max(m)); 
		dir=1;
		sugDel=meandif(m);
	else
		sugQ(1)=mmih-std(min(m')); 
		sugQ(2)=mmah+std(max(m')); 
		dir=2;
		sugDel=meandif(m');
	end;
	sugQ=rounddef(sugQ,3);
	sugDel=rounddef(sugDel,2);
	

function y=rounddef(x,n);
% function rounddef(x,n);
% rounds x to n decimal places
sg=sign(x);
ind=find(sg~=0);
lx=ceil(log10(abs(x(ind))))-n;
dlx=10.^lx;
y=zeros(size(x));
y(ind)=round(x(ind)./dlx).*dlx;
