% Macro for plotting PSD spectra saved in SPEC file at ID10B
%* An pattern file for GID/GISAX on liquid interface at ID10B, beam
%		deflected from the horizontal plane by mirrors in the optical hutch
% (c) 2007, Jiri Novak, ESRF Grenoble, France 
%%%%
% This program comes with ABSOLUTELY NO WARRANTY.  This is free
% software distributed under GNU licence, see
% http://www.gnu.org/licenses/ or the attached file GPLicense.txt



%% Flags - mark what do you want to do, 0/1=no/yes, operations are
% ordered in sequence, which should be used to plot your data.
% Skipping one of operations in the sequence can cause an error.

flgUnspec=1; % Extract data from a SPEC file to Matlab
flgNormalize=1; % Normalize intensities for monitor and absorbers 
flgAng2Q=0; % Transform from the angular space to the reciprocal space
flgPlotPSD=0; % Plot rough PSD spectra
flgAutoGridEst=0; % Should I perform automatic estimation of the density
									%	and size of the grid?
flgGridQ=0;	% Perform gridding (mapping of data in Q space from unevenly
						% distributed measurement points to a regular grid)
flgPlotGrid=0; % Plot data in Q space (gridded data) 
flgPrintPlot=0; % Print plot currently plotted to a pdf file
flgSaveMat=1; % Save the scan data to a mat file (matlab native format)
flgLoadMat=0; % Load the scan data from a mat file previously saved
							% (speeds up the process of plotting, whenever extracting
							% and gridding of data was already done in a previous
							% session)
flgSaveAscii=0; % Export data into ASCII files 

%% Choices:
% File-name of the spec-file
 SpecName='./MSpScanLiquid.spec';
% Name of the variable which should contain scan data within your Matlab
% session. The type of the variable  will be a structure. 
%		(a structure is a type of variable like double or integer)
scn='s25a26';
% Scan(s) number. Several scans can be concatenated into one by
% 	something like scNum=[11 12 13];
scNum=[25 26];
%%Correct SPEC values
% Give a cell array (closed in {}) of motor names and their offsets,
% which should be substracted from the values in the SPEC file
% MatlabValue=SPEC_Value-MotDetOff{2*i-1}
MotDetOff={'delta',0,'gamma',44.7};
% Names of detectors to be normalized for attenuators and monitor. PSD
% stands for the name of the variable with the PSD spectra (an array of
% intensities in each channel for each measurement point)
NameDet={'all1','PSD'};
% Name of motor, which drives the attenuator
NameAtt='curratt';
% Name of monitor
NameMon='Monitor';
% Normalization factor for monitor (approximate average value over
% 	several years:)
MonNorm=1e5;
NormFac=3.892; % Attenuation factor for one filter 
lambda=1.55; % 8 keV

%% Part: Transformation to the reciprocal space
%				The transformation function depends on the experimental setup -
%				see below.  The transformation function (an .m file) should
%				accept the scan structure (the fields are the motor names and
%				intensities on the detectors, MCA ...) as the first parameter.
%       The transformation function adds fields Qx,Qy,Qz with calculated
%       reciprocal space coordinates.
% 			The prescribed form of : 
%			  Ang2Q={'mfile_name','motor1','motor2','motor3',...};
%% Setup types:
%*	GID/GISAX/reflectivity on liquid interface at ID10B,
%   i.e., sample plane is horizontal (liquid), the primary beam is
%   deflected from the horizontal plane by the deflection crystal by an
%   angle mu (i.e.  mu= angle of incidence) or by  mirrors in the
%   optical hutch by an angle alfi (then alfi should be defined in this
%   macro). The PSD detector is rotated around the vertical axis by an
%   angle del.  The detector is rotated around the horizontal axis by an
%   angle gam and gam sits on del, PSD detector is oriented along the
%   vertical axis.
% 	Motors: NameAi='mu', NameDel='del', NameGam='gam'
%		Ang2Q={'MSpAng2Q1v2',NameAi,NameDel,NameGam};
%	  In particular on ID10B ESRF (uncomment in case of need):	
%		Ang2Q={'MSpAng2Q1v2','mu','delta','gamma'};

%%  For ID10B ESRF: 
%		In case of deflection of the beam from the horizontal plane by
%   mirrors in the optical hutch by an angle afli, uncomment (i.e.
%   remove the % character at the beginning of the line) and edit
%   following lines
		alfi=.1127;
		Ang2Q={'MSpAng2Q1v2','alfi','delta','gamma'};

%* 	Scattering geometry with a sample on the horizontal stage at
% 	ID10B (GID, GISAX, reflectivity, coplanar diffraction).  The sample
% 	surface plane is tilted by an angle chigH with respect to the
% 	horizontal plane (plane of the beam). The detector arm is rotated
% 	around the vertical axis by an angle del and  around the horizontal
% 	axis by an angle gam. The PSD detector is oriented along the
% 	vertical axis for del=gam=0.  Motor gam sits on the motor del.
%	  In particular on ID10B ESRF (uncomment in case of need):	
%		Ang2Q={'MSpAng2Q2v2','chigH','delta','gamma'};

% c0 ... central channel of the position sensitive detector (PSD), i.e.
% chanel to which direct beam impignes, when the PSD is at the 0
% position in terms of detector arm motors
c0=71.5;
% cpd ... channels per degree of PSD
cpd=71.57;
% Range of interest ROI (i.e. range of the pixels which should be loaded
% from the spec file. Form ROI=[FirstPixel,LastPixel]
ROI=[90,910];

%% Part: Gridding of the intensity

% Grid points and corresponding widths of the bins for gridding of the
% PSDNorm in Q space-> result given int the field PSDNormg.
% You can use flgAutoGrid=1; to estimate density of the grid-points, the
% size of the grid in the reciprocal space and length of the sides 
% of parallelepiped delQ around grid-points. 
Qxg=[-.49]; delQx=1;
Qyg=[1.05:0.015:2.1]; delQy=0.015;
Qzg=[0:1e-3:1.2]; delQz= .001;

%% Part: The 2D plot:

% Which wariable will be on x- and y-axis in the 2D plot of the gridded
% data?
xGridField='Qyg';
yGridField='Qzg';
colorGridField='PSDNormg'; % Which variable is colorscaled (z-variable)

%% PSD plot (rough data plotting = a motor on x-axis, channel number on the y-axis)
PSDPlotXField='delta'; % variable on the x-axis of the 2D plot
PSDPlotPSD='PSDNorm'; % which variable is colorscaled (z-variable)
%% Plot of the gridded data

flgIntLog=1; % Plot intensity logarithmically? 0/1=no/yes
truncLogBot=0; % Maximum of log10(intensity) which will be colorscaled
							 % all values higher than
							 % max(max(intensity))*10^(-truncLogBot) will be
							 % truncated, i.e. colorscaled to this value
truncLogTop=0.0; % Minimum of log10(intensity) which will be colorscaled
							 % All values smaller than
							 % max(max(intensity))*10^(-truncLogBot) will be
							 % truncated, i.e. colorscaled to this value
							 % Take care truncLogBot>truncLogTop

AxFontSize=12;
LabelFontSize=12;
%%Saving options
% FName ... Root of the filename for saving images and scan data into a
% matlab mat file or ASCII files
FName='MSpScanLiquid';
% Print format for images: ps,psc,ps2,psc2,eps,epsc,eps2,epsc2,jpeg,png
PrintFormat='jpeg'; 
% SaveASCII_Vars ... variables (motors, detectors, ...), which should be
% 	saved into ASCII files (the flgSaveAscii option)
%   Some  variables are: 
%   PSD, PSDNorm ... intensities on the linear detector rough/normalized
%		Qx,Qy,Qz ... reciprocal space coordinates
%		Qxg,Qyg,Qzg, PSDNormg ... grid points in reciprocal space + gridded
%			intensity
%		channs ... order numbers of the channnels loaded to Matlab in PSD
%			variable
%   Example: SaveASCII_Vars={'Qxg','Qyg','PSDNormg'};
%						 SaveASCII_Vars={'delta','channs','PSDNorm'};
SaveASCII_Vars={'delta','channs','PSDNorm'};

% J. Novak, ID10B ESRF Grenoble, France,  
%%%%
% This program comes with ABSOLUTELY NO WARRANTY.  This is free
% software distributed under GNU licence, see
% http://www.gnu.org/licenses/ or the attached file GPLicense.txt


% 	e-mail: novak@esrf.fr
% 2007-11-20
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!%
MSpMain2
