function intgrid=gridq1_01(qxg,qyg,qzg,oint,qx,qy,qz,wx,wy,wz,method);
% gridq.m
% Grids data
% function intgrid=gridq1_01(qxg,qyg,qzg,oint,qx,qy,qz[,wx,wy,wz,method]);
% qxg, qyg, qzg ... grid points in reciprocal space must be regular
% and in increasing order
% oint ... intensity
% qx qy qz ... reciprocal space coordinates of points in oint
% wx, wy, wz ... width of the bins around the grid points,
% min(diff(qxg)) min(diff(qyg)) min(diff(qzg)) by default
% intgrid ... grided data, qxg changes along columns, qyg along rows 
%   qyg along 3 dimension
% method mean/nearest
% 	* mean (default)= average data over boxes of the widths wx,wy,wz around grid
% 		point
%	  * nearest= put value from the nearest point which is still within box
%		of the width wx,wy,wz around the grid point
% This function is interface to c program gridq1_01
% (c) 2007, Jiri Novak, ESRF Grenoble, France
 
%%%%
% This program comes with ABSOLUTELY NO WARRANTY.  This is free
% software distributed under GNU licence, see
% http://www.gnu.org/licenses/ or the attached file GPLicense.txt



nx= length(qxg);
ny= length(qyg);
nz= length(qzg);
if nx>1,
	dels=diff(qxg);
	delmi= min(dels); delma= max(dels);
  if abs(delmi-delma)<4*eps(max(abs(qxg))),
		delx=delmi;
	else
		error('Grid points on x-axis are not regulary spaced\n');
	end;
  if any(dels<0),
		error('Decreasing sequence of grid-points on x-axis\n');
	end;
else
  delx= 1;
end;
if ny>1, 
	dels=diff(qyg);
	delmi= min(dels); delma= max(dels);
  if abs(delmi-delma)<4*eps(max(abs(qyg))),
		dely=delmi;
	else
		error('Grid points on y-axis are not regulary spaced\n');
	end;
  if any(dels<0),
		error('Decreasing sequence of grid-points on y-axis\n');
	end;
else
  dely=1;
end;
if nz>1,
	dels=diff(qzg);
	delmi= min(dels); delma= max(dels);
  if abs(delmi-delma)<4*eps(max(abs(qzg))),
		delz=delmi;
	else
		error('Grid points on z-axis are not regulary spaced\n');
	end;
  if any(dels<0),
		error('Decreasing sequence of grid-points on z-axis\n');
	end;
else
  delz=1;
end;
if nargin<8,
  wx=delx;
  wy=dely;
  wz=delz;
end;
if nargin<11,
	method='mean';
end;
% Write input file
fid=fopen('grid_tmp.inp','w');
fprintf(fid,'$%d #number of datapoints\n',prod(size(qx)));
fprintf(fid,['$real*4 qxtmp.b %g %g %d %g  # file_with_qx qx_start' ...
	' number_of_grid_points_on_qx delta-qx' ...
  ' qx_bin_width\n'],qxg(1),delx,nx,wx);
fprintf(fid,['$real*4 qytmp.b %g %g %d %g  # file_with_qy qy_start',...
	' number_of_grid_points_on_qy delta-qy', ...
  ' qy_bin_width\n'],qyg(1),dely,ny,wy);
fprintf(fid,['$real*4 qztmp.b %g %g %d %g  # file_with_qz qz_start', ...
	' number_of_grid_points_on_qz delta-qz' ...
  ' qz_bin_width\n'],qzg(1),delz,nz,wz);
fprintf(fid,'$real*4 ointtmp.b #input intensity\n');
fprintf(fid,'$real*4 ginttmp.b #gridded intensity, output\n');
fprintf(fid,'$%s\n',method);
fclose(fid);
%Write qx,qy,qz,oint
fid=fopen('qxtmp.b','wb');
fwrite(fid,qx,'float32');
fclose(fid);
fid=fopen('qytmp.b','wb');
fwrite(fid,qy,'float32');
fclose(fid);
fid=fopen('qztmp.b','wb');
fwrite(fid,qz,'float32');
fclose(fid);
fid=fopen('ointtmp.b','wb');
fwrite(fid,oint,'float32');
fclose(fid);
unix('gridq1_01 grid_tmp.inp');
fid=fopen('ginttmp.b','rb');
intgrid=fread(fid,nx*ny*nz,'float32');
fclose(fid);
intgrid=reshape(intgrid,nx,ny,nz);



